import java.util.*;

public class Esame{
  
  
  private static class Veicolo implements Comparable<Veicolo>{
    public double uso; //ustilizzo in km
    public String targa; //targa veicolo
    public String tipo; //tipologia carburante

	/**
	* Costruttore con 3 argomenti
	*
	* @param t targa del veicolo
	* @param f tipologia carburante
	* @param km uso (in km)
	*/
    public Veicolo(String t, String f, double km){
      targa = t;
      tipo = f;
      uso = km;	
    }

    public String toString(){
	    return "<"+targa+", "+tipo+", "+uso+">";
    }


    /**
    * Implementa il metodo compareTo per oggetti di tipo Veicolo. (Punti 25)
    * Un veicolo v1 e' minore (o maggiore) di un altro veicolo v2 se l'uso di v1 e' minore (o maggiore) della durata di v2. 
    * Se i veicoli v1 ed v2 hanno stesso uso il veicolo v1 e' minore del veicolo v2 se la targa di v1 precede nell'ordine lessicografico la targa del veicolo v2. Altrimenti v2 e' maggiore di v1.
    * Se i veicoli v1 e v2 hanno stesso uso e stessa targa il veicolo v1 e' minore del veicolo v2 se il tipo di v1 precede nell'ordine lessicografico il tipo di v2. Altrimenti il veicolo v2 e' maggiore del veicolo v1.
    * Se i veicolo v1 ed v2 hanno stesso uso, stessa targa e stesso tipo allora sono uguali. 
    * @param b tipo con cui comparare l'oggetto di invocazione
    * @return un intero positivo/negativo/nullo se l'oggetto di invocazione e' minore/maggiore/uguale al parametro.
    */
    public int compareTo(Veicolo v){
			int diff = Integer.signum((int)(uso-v.uso)); 
			if (diff!=0) {
				return diff;
			} else {
				return (targa.compareTo(v.targa)==0)?Integer.signum(tipo.compareTo(v.tipo)):Integer.signum(targa.compareTo(v.targa)); 
						
			}
    }

    public boolean equals(Object o){
	if (o instanceof Veicolo) {
		Veicolo v = (Veicolo) o;
		return (targa.equals(v.targa))&&(tipo.equals(v.tipo))&&(uso==v.uso);
	}
	return false;	
    }	

    public int hashCode(){
	return targa.hashCode() + tipo.hashCode() + (new Double(uso)).hashCode();
    }	
	
 } 

  
  /**
  * data una matrice quadrata di double, ritorna true se la somma della diagonale secondaria è un valore strettamente maggiore di zero, false altrimenti (Punti 25) 
  * Ad esempio data la matrice 
  * {{ 0.3, 0.7, -0.8 },
  *  { -0.5, 0.9, -0.5 },
  *  { 0.2, 0.1, 0.0 }}
  * il metodo ritorna
  * 	  true (perchè la somma 0.2+0.9-0.8 = 0.3 > 0)
  * se la matrice non contiene nessun elemento il metodo ritorna false.
  *
  *  @param m la matrice di double in ingresso
  *  @return true se la somma della diagonale secondaria è un valore strettamente maggiore di zero, false altrimenti 
  */	
  public static boolean posSecDiag(double[][] m){
	double acc = 0;
	for (int i=0; i<m.length;i++){		
		acc += m[i][m.length-1-i];
	}
    	return (acc>0);
  }
  
 
  /**
  * Data una mappa che associa un veicolo ad un'area geografica, ritornare una mappa che associa ad ogni area geografica presente nella mappa il veicolo con uso minore che appartiene a quella area (Punti 20)
  * Ad esempio se m = {<"FS123HB", "metano", 2350>="Verona Sud", <"RM555MR", "Benzina", 1500>="Verona Sud", <"VR777RV","Diesel", 2500>="Verona Sud", <"VI666IV","Benzina",1250>="Verona Est"} la mappa da restituire sara' {"Verona Sud"= <"RM555MR", "Benzina", 1500>, "Verona Est" = <"VI666IV","Benzina",1250>}.
  * il metodo deve ritornare una mappa vuota se la mappa di partenza e' vuota.
  *
  * @param m mappa che associa veicoli a zone
  * @return una mappa che associa ad ogni area geografica presente nella mappa il veicolo con uso minore che appartiene a quella area.
  */
  public static Map<String,Veicolo> zoneToVehicle(Map<Veicolo,String> m){
    
      Map<String, Veicolo> res = new HashMap<String, Veicolo>();
	for (Veicolo v : m.keySet()){
	     String currZone = m.get(v);	
	     if ( res.keySet().contains(currZone) ) {
		if (v.uso<res.get(currZone).uso){		
			res.put(currZone,v);	
		}
	     } else {
		res.put(currZone,v);
	     }
    	}	
    return res;
  }

  /**
  * Dato un grafo e due nodi src e dst ritornare true se dst è raggiungibile da src e false altrimenti (Punti 30)
  * 
  * @param g grafo in ingresso
  * @param src nodo di partenza
  * @param dst nodo da raggiungere
  * @return true se dst è raggiungibile da src false altrimenti
  */
  public static boolean reach(Grafo g, Object src, Object dst){
	Set<Object> visited = new HashSet<Object>();
	if (g.getEdgeSet().size()==0){
		return false;
	}
	dfs(g,src,visited);	
    	return visited.contains(dst);
  }

  public static void dfs(Grafo g, Object node, Set<Object> visited){
	visited.add(node);
	for(Arco a : g.getEdgeSet(node)){
		if (!visited.contains(a.getNode1())){
			dfs(g,a.getNode1(),visited);		
		}
		if (!visited.contains(a.getNode2())){
			dfs(g,a.getNode2(),visited);		
		}
	}
  }

  public static void main(String[] args){

 
    System.out.println("**********TEST compareTo per Veicolo*************");	

	Veicolo fs = new Veicolo("FS123HB", "metano", 2350);
	Veicolo rm = new Veicolo("RM555MR", "Benzina", 1500);
	Veicolo vr = new Veicolo("VR777RV","Diesel", 2500);
	Veicolo vi = new Veicolo("VI666IV","Benzina",1250);
	Veicolo test = new Veicolo("AA111AA","Benzina",1250);
	Veicolo test2 = new Veicolo("AA111AA","Benzina",1250);

	System.out.println("Veicolo "+fs+" comparato con "+rm+" "+fs.compareTo(rm)+" [dovrebbe essere positivo])");

	System.out.println("Veicolo "+fs+" comparato con "+vr+" "+fs.compareTo(vr)+" [dovrebbe essere negativo])");

	System.out.println("Veicolo "+test+" comparato con "+vi+" "+test.compareTo(vi)+" [dovrebbe essere negativo])");

	System.out.println("Veicolo "+test+" comparato con "+test2+" "+test.compareTo(test2)+" [dovrebbe essere nullo])");

    
    System.out.println("**********TEST posSecDiag()*************");

	double[][] m1= {
	{ 0.3, 0.7, -0.8 },
  	{ -0.5, 0.9, -0.5 },
  	{ 0.2, 0.1, 0.0 }
    };

	double[][] m2= {
      { -0.5, -0.9},
      { 0.3, 0.3},
    };

	double[][] m3= new double[0][0];


	boolean res1 = posSecDiag(m1);
        System.out.println("diagonale secondaria a somma positiva "+Arrays.deepToString(m1)+" ?\n "+res1+" dovrebbe essere True");

	boolean res2 = posSecDiag(m2);
        System.out.println("diagonale secondaria a somma positiva "+Arrays.deepToString(m2)+" ?\n "+res2+" dovrebbe essere False");

	boolean res3 = posSecDiag(m3);
        System.out.println("diagonale secondaria a somma positiva "+Arrays.deepToString(m3)+" ?\n "+res3+" dovrebbe essere False");
      
    System.out.println("**********TEST zoneToVehicle()*************");
 

	Map<Veicolo,String> map1 = new HashMap<Veicolo,String>();
	map1.put(fs,"Verona Sud"); map1.put(rm,"Verona Sud"); map1.put(vr,"Verona Sud"); map1.put(vi,"Verona Est"); 

	Map<Veicolo,String> map2 = new HashMap<Veicolo,String>();
	
        System.out.println("mappa in ingresso "+map1+" \n mappa in uscita "+zoneToVehicle(map1)+" dovrebbe essere {Verona Sud= <RM555MR, Benzina, 1500>, Verona Est = <VI666IV,Benzina,1250>}");
	System.out.println();

        System.out.println("mappa in ingresso "+map2+" \n mappa in uscita "+zoneToVehicle(map2)+" dovrebbe essere {}");
	System.out.println();
	
    System.out.println("**********TEST reach()*************");
 
	    Object na = "a"; Object nb = "b"; Object nc = "c"; Object nd = "d"; Object ne = "e"; Object nf = "f"; Object nh = "h"; Object nk = "k";

	    Grafo g = new Grafo("g");    
	    g.add(na,nb,0);
	    g.add(nb,nc,0);
	    g.add(nc,nd,0);
	    g.add(nc,ne,0);
	    g.add(nh,nk,0);
	    
	    System.out.println("L'insieme di archi per il grafo g: " + g.getEdgeSet());
		

	   System.out.println("Nodo d raggiungibile dal nodo a " + reach(g,na,nd) +" dovrebbe essere True");


	   System.out.println("Nodo k raggiungibile dal nodo a " + reach(g,na,nk) +" dovrebbe essere False");

    
  }

}
